local Tablet = Rock("Tablet-2.0", false, true)

RaidSaveFu = Rock:NewAddon("RaidSaveFu", "LibRockDB-1.0", "LibRockEvent-1.0", "LibRockConfig-1.0", "LibRockConsole-1.0", "LibFuBarPlugin-3.0")
local RaidSaveFu = RaidSaveFu
RaidSaveFu.title = "RaidSaveFu"
RaidSaveFu:SetDatabase("RaidSaveFuDB");

RaidSaveFu:SetDatabaseDefaults('profile', {
	textwhenzero = false,
	shorttext = false,
	showid = true,
} )
RaidSaveFu:SetFuBarOption('tooltipType', "Tablet-2.0")
RaidSaveFu:SetFuBarOption('iconPath', "Interface\\Addons\\FuBar_RaidSaveFu\\icon.tga")
RaidSaveFu:SetFuBarOption('hasNoColor', true)
RaidSaveFu:SetFuBarOption('cannotHideText', false)
--RaidSaveFu:SetFuBarOption('clickableTooltip', true)

function RaidSaveFu:OnInitialize()
	local optionsTable = {
		name = "FuBar_RaidSaveFu",
		desc = self.notes,
		handler = RaidSaveFu,
		type = 'group',
		args = {
			twentyFour = {
				type = 'toggle',
				name = "Hide text for no saved instances",
				desc = "Show or hide the FuBar text when the character has no saved instances",
				get = "IsTextWhenZero",
				set = "ToggleTextWhenZero",
			},
			showID = {
				type = 'toggle',
				name = "Show instance IDs",
				desc = "Show or hide the numeric instance IDs",
				get = "IsShowID",
				set = "ToggleShowID",
			},
			shortText = {
				type = 'toggle',
				name = "Short FuBar text",
				desc = "Only show the number of saved instances (no extra text)",
				get = "IsShortText",
				set = "ToggleShortText",
			},
		}
	}
	RaidSaveFu:SetConfigTable(optionsTable)
	RaidSaveFu:SetConfigSlashCommand("/raidsavefu")
	RaidSaveFu.OnMenuRequest = optionsTable
	if self.db.profile.realm == nil then
		self.db.profile.realm = { }
	end
	if self.db.profile.realm[GetRealmName()] == nil then
		self.db.profile.realm[GetRealmName()] = { }
	end
	if self.db.profile.realm[GetRealmName()].char == nil then
		self.db.profile.realm[GetRealmName()].char = { }
	end
	if self.db.profile.realm[GetRealmName()].name == nil then
		self.db.profile.realm[GetRealmName()].name = GetRealmName()
	end
	if self.db.profile.realm[GetRealmName()].char[GetUnitName("player")] == nil then
		self.db.profile.realm[GetRealmName()].char[GetUnitName("player")] = { }	
	end
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	if player.numsaved == nil then
		player.numsaved = 0
	end
	if player.instance == nil then
		player.instance = { }
	end
	player.name = GetUnitName("player")
	player.class = strupper(UnitClass("player"))
end

function RaidSaveFu:OnEnable()
	self:AddEventListener("UPDATE_INSTANCE_INFO", "Refresh")
	self:AddEventListener("PLAYER_ENTERING_WORLD", "RequestRefresh")
	self:AddEventListener("CHAT_MSG_SYSTEM", "CheckSystemMessage")
end

function RaidSaveFu:OnDisable()
	self:RemoveEventListener("UPDATE_INSTANCE_INFO")
	self:RemoveEventListener("PLAYER_ENTERING_WORLD")
	self:RemoveEventListener("CHAT_MSG_SYSTEM")
end

function RaidSaveFu:OnUpdateFuBarText()
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	local fubartext
	if self:IsShortText() then
		fubartext = string.format("%d", player.numsaved)
	elseif player.numsaved == 1 then
		fubartext = string.format("%d Saved Instance", player.numsaved)
	else
		fubartext = string.format("%d Saved Instances", player.numsaved)
	end
	
	if player.numsaved == 0 and self:IsTextWhenZero() then
		self:SetFuBarText("");
	else
		self:SetFuBarText(fubartext)
	end
end

function RaidSaveFu:OnUpdateFuBarTooltip()
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	if player.numsaved ~= 0 then
		local displaytable = { }
		local i
		for i = 1, player.numsaved do
			if not self:IsPastTimestamp(player.instance[i].resetdate) then
				local namestring = player.instance[i].name
				if self:IsShowID() then
					namestring = namestring..GRAY_FONT_COLOR_CODE.." ["..player.instance[i].id.."]|r"
				end
				local displayrow = { name = namestring, id = player.instance[i].id, resetdate = player.instance[i].resetdate }
				table.insert(displaytable, displayrow)
			end
		end
		if # displaytable ~= 0 then
			local cat = Tablet:AddCategory(
				'columns', 2,
				'text', self:GetClassColorCode(player.class)..player.name.."|r",
				'justify', "LEFT",
				'justify2', "RIGHT"
			)
			cat:AddLine(
				'text', HIGHLIGHT_FONT_COLOR_CODE.."Instance|r",
				'text2', HIGHLIGHT_FONT_COLOR_CODE.."Resets (approx.)|r"
			)
			local i
			for i = 1, # displaytable do
				local namestring = displaytable[i].name
				cat:AddLine(
					'text', namestring,
					'text2', displaytable[i].resetdate
				)
			end
		end
	end
	local char = self.db.profile.realm[GetRealmName()].char
	local k, v
	for k, v in pairs(char) do
		if v.numsaved ~= 0 and v.name ~= player.name then
			local displaytable = { }
			local i
			for i = 1, v.numsaved do
				if not self:IsPastTimestamp(v.instance[i].resetdate) then
					local namestring = v.instance[i].name
					if self:IsShowID() then
						namestring = namestring..GRAY_FONT_COLOR_CODE.." ["..v.instance[i].id.."]|r"
					end
					local displayrow = { name = namestring, id = v.instance[i].id, resetdate = v.instance[i].resetdate }
					table.insert(displaytable, displayrow)
				end
			end
			if # displaytable ~= 0 then
				local cat = Tablet:AddCategory(
					'columns', 2,
					'text', self:GetClassColorCode(v.class)..v.name.."|r",
					'justify', "LEFT",
					'justify2', "RIGHT"
				)
				cat:AddLine(
					'text', HIGHLIGHT_FONT_COLOR_CODE.."Instance|r",
					'text2', HIGHLIGHT_FONT_COLOR_CODE.."Resets (approx.)|r"
				)
				local i
				for i = 1, # displaytable do
					local namestring = displaytable[i].name
					cat:AddLine(
						'text', namestring,
						'text2', displaytable[i].resetdate
					)
				end
			end
		end
	end
--	Tablet:SetHint("Click to force values to be refreshed")
end

--function RaidSaveFu:OnFuBarClick(button)
--	self:RequestRefresh()
--end

function RaidSaveFu:CheckSystemMessage(arg)
	if tostring(arg) == "INSTANCE_SAVED" then
		self:RequestRefresh()
	end
end

function RaidSaveFu:RequestRefresh()
	RequestRaidInfo()
end

function RaidSaveFu:Refresh()
	local player = self.db.profile.realm[GetRealmName()].char[GetUnitName("player")]
	local expires2reset = { } -- cache results to try to save computations
	player.numsaved = GetNumSavedInstances()
	player.instance = { }
	if player.numsaved == 0 then
	else
		local iname, iid, iexpires, iresetdate, temp
		for i = 1, player.numsaved do
			iname, iid, iexpires = GetSavedInstanceInfo(i)
			if expires2reset[iexpires] == nil then
				local ryear, rmonth, rday, rhour, rmin = self:ExpiresToDate(iexpires)
				iresetdate = string.format("%d-%02d-%02d %02d:%02d", ryear, rmonth, rday, rhour, rmin)
				table.insert(expires2reset, iexpires, iresetdate)
			else
				iresetdate = expires2reset[iexpires]
			end	
			temp = { name = iname, id = iid, resetdate = iresetdate }
			table.insert(player.instance, temp)
		end
	end
--	self:Print("instance information refreshed")
	self:UpdateFuBarPlugin()
end

function RaidSaveFu:IsPastTimestamp(timestamp)
	local year, month, day, hours, minutes, now
	year = tonumber(date("%Y"))
	month = tonumber(date("%m"))
	day = tonumber(date("%d"))
	hours, minutes = GetGameTime()
	now = string.format("%d-%02d-%02d %02d:%02d", year, month, day, hours, minutes)
	return timestamp <= now
end

function RaidSaveFu:ExpiresToDate(seconds)
	local year, month, day, hours, minutes, temp
	year = tonumber(date("%Y"))
	month = tonumber(date("%m"))
	day = tonumber(date("%d"))
	hours, minutes = GetGameTime()
	temp = seconds - (seconds % 60) -- now just the seconds, correct to the minute
	temp = temp / 60 -- number of minutes still active
	while temp > 0 do
		minutes = minutes + 1
		if minutes > 59 then
			minutes = 0
			hours = hours + 1
			if hours > 23 then
				hours = 0
				day = day + 1
				if day > self:GetDaysInMonth(year, month) then
					day = 1
					month = month + 1
					if month > 12 then
						month = 1
						year = year + 1
					end
				end
			end
		end
		temp = temp - 1
	end
	if (seconds % 60) > 29 then
		minutes = minutes + 1
		if minutes > 59 then
			minutes = 0
			hours = hours + 1
			if hours > 23 then
				hours = 0
				day = day + 1
				if day > self:GetDaysInMonth(year, month) then
					day = 1
					month = month + 1
					if month > 12 then
						month = 1
						year = year + 1
					end
				end
			end
		end
		temp = temp - 1
	end
	return year, month, day, hours, minutes
end

function RaidSaveFu:GetDaysInMonth(year, month)
	local function is_leap_year(year)
		return year % 4 == 0 and (year % 100 ~= 0 or year % 400 == 0)
	end
	return month == 2 and is_leap_year(year) and 29 or ("\31\28\31\30\31\30\31\31\30\31\30\31"):byte(month)
end

function RaidSaveFu:GetClassColorCode(class)
	local r = string.format("%x", math.floor(RAID_CLASS_COLORS[class].r * 255))
	local g = string.format("%x", math.floor(RAID_CLASS_COLORS[class].g * 255))
	local b = string.format("%x", math.floor(RAID_CLASS_COLORS[class].b * 255))
	return "|cff"..r..g..b
end

-- ** functions for the UI configuration below **

function RaidSaveFu:IsTextWhenZero()
	return self.db.profile.textwhenzero
end

function RaidSaveFu:ToggleTextWhenZero()
	self.db.profile.textwhenzero = not self.db.profile.textwhenzero
	self:UpdateFuBarText()
	return self.db.profile.textwhenzero
end

function RaidSaveFu:IsShortText()
	return self.db.profile.shorttext
end

function RaidSaveFu:ToggleShortText()
	self.db.profile.shorttext = not self.db.profile.shorttext
	self:UpdateFuBarText()
	return self.db.profile.shorttext
end

function RaidSaveFu:GetDurationFormat()
	return self.db.profile.durationformat	
end

function RaidSaveFu:SelectDurationFormat(value)
	self.db.profile.durationformat = value
	self:UpdateFuBarTooltip()
	return self.db.profile.durationformat	
end

function RaidSaveFu:IsColorfulDuration()
	return self.db.profile.colorfulduration
end

function RaidSaveFu:ToggleColorfulDuration()
	self.db.profile.colorfulduration = not self.db.profile.colorfulduration
	self:UpdateFuBarTooltip()
	return self.db.profile.colorfulduration
end

function RaidSaveFu:IsShowID()
	return self.db.profile.showid
end

function RaidSaveFu:ToggleShowID()
	self.db.profile.showid = not self.db.profile.showid
	self:UpdateFuBarTooltip()
	return self.db.profile.showid
end
